@@ -1,5 +1,7 @@
 estfun.lmerMod <- function(x, ...) {
   if (!is(x, "lmerMod")) stop("estfun.lmerMod() only works for lmer() models.")
+  if (!is.null(x@call$weights)) stop ("Models with weights specification is currently not supported.")
+  if (length(grep("cbind", x@call$formula))!=0) stop ("Models with cbind specification is currently not supported.")
   
   ## warning for high correlations. 
   cor <- attr(lme4::VarCorr(x)[[1]], "correlation")
