@@ -1,4 +1,5 @@
 vcov.lmerMod <- function(object, ...) {
+  
   if (!is(object, "lmerMod")) stop("estfun.lmerMod() only works for lmer() models.")
   
   dotdotdot <- list(...)
@@ -24,7 +25,7 @@ vcov.lmerMod <- function(object, ...) {
   ## total variance
   uluti <- length(parts$theta)
   Zlam <- tcrossprod(parts$Z, parts$Lambdat)
-  V <- (tcrossprod(Zlam, Zlam) + diag(1, parts$n)) * (parts$sigma)^2
+  V <- (tcrossprod(Zlam, Zlam) + Diagonal(parts$n, 1)) * (parts$sigma)^2
   M <- solve(chol(V))
   invV <- tcrossprod(M, M)
   LambdaInd <- parts$Lambda
@@ -32,97 +33,88 @@ vcov.lmerMod <- function(object, ...) {
   invVX <- crossprod(parts$X, invV)
   Pmid <- solve(crossprod(parts$X, t(invVX)))
   P <- invV - tcrossprod(crossprod(invVX, Pmid), t(invVX))
-    
+  
   ## block 1 fixhes
   fixvar <- solve(tcrossprod(crossprod(parts$X, invV), t(parts$X)))
   if (full == FALSE) {
     fixvar  
   } else {
     fixhes <- tcrossprod(crossprod(parts$X, invV), t(parts$X))
-  
+    
     ## length = (variance covariance parameter in G + residual variance.)
     uluti <- length(parts$theta)
     devV <- vector("list", (uluti + 1))
-  
+    
     ## get d(G)/d(sigma), faciliated by d(Lambda). 
     devLambda <- vector("list", uluti)
     score_varcov <- matrix(NA, nrow = length(parts$y), ncol = uluti)
     for (i in 1:uluti) {
       devLambda[[i]] <- forceSymmetric(LambdaInd==i, uplo = "L")
-      devV[[i]] <- tcrossprod(tcrossprod(parts$Z, t(devLambda[[i]])),
-        parts$Z)
+      devV[[i]] <- tcrossprod(tcrossprod(parts$Z, t(devLambda[[i]])), parts$Z)
     }
-    devV[[(uluti+1)]] <- diag(1, nrow(parts$X))
-  
+    devV[[(uluti+1)]] <- Diagonal(nrow(parts$X), 1)
+    
     ## Block 4: sigma's second derivative.
     ranhes <- matrix(NA, nrow = (uluti + 1), ncol = (uluti + 1))
     ## combinations (allow repeat) to indicate entries
     entries <- rbind(matrix(rep(1: (uluti + 1), each = 2),
-      (uluti + 1), 2, byrow = TRUE), t(combn((uluti + 1), 2)))
-      entries <- entries[order(entries[,1], entries[,2]), ]
+                            (uluti + 1), 2, byrow = TRUE), t(combn((uluti + 1), 2)))
+    entries <- entries[order(entries[,1], entries[,2]), ]
     ## ML estimates
     if (object@devcomp$dims[10] == 0) {
       if(information == "expected") {
-        for (i in 1: nrow(entries)) {
-          ranhes[lower.tri(ranhes, diag = TRUE)][i] <- as.numeric((1/2) *
-            sum(diag(tcrossprod(tcrossprod(crossprod(invV,
-            devV[[entries[i,1]]]), invV), t(devV[[entries[i,2]]])))))
-        }
+        ranhes[lower.tri(ranhes, diag = TRUE)] <- 
+          apply(entries, 1, function(x) as.numeric((1/2) *
+                lav_matrix_trace(tcrossprod(tcrossprod(crossprod(invV,
+                devV[[x[1]]]), invV), t(devV[[x[2]]])))))
       }
-       if(information == "observed") {
-        for (i in 1: nrow(entries)) {
-          ranhes[lower.tri(ranhes, diag = TRUE)][i] <- -as.numeric((1/2) *
-            sum(diag(tcrossprod(tcrossprod(crossprod(invV,
-            devV[[entries[i,1]]]), invV), t(devV[[entries[i,2]]]))))) +
-              tcrossprod((tcrossprod((crossprod(yXbe,
-              tcrossprod(tcrossprod(crossprod(invV,
-              devV[[entries[i,1]]]), invV), t(devV[[entries[i,2]]])))),
-              invV)), t(yXbe))
-        }
+      if(information == "observed") {
+        ranhes[lower.tri(ranhes, diag = TRUE)] <- apply(entries, 1, 
+          function(x) -as.numeric((1/2) *
+          lav_matrix_trace(tcrossprod(tcrossprod(crossprod(invV,
+          devV[[x[1]]]), invV), t(devV[[x[2]]])))) +
+          tcrossprod((tcrossprod((crossprod(yXbe,
+          tcrossprod(tcrossprod(crossprod(invV,
+          devV[[x[1]]]), invV), t(devV[[x[2]]])))), invV)), t(yXbe)))
       }      
     }
     ## REML estimates
     if (object@devcomp$dims[10] == 2|object@devcomp$dims[10] == 1) {
       if(information == "expected") {
-        for (i in 1: nrow(entries)) {
-          ranhes[lower.tri(ranhes, diag = TRUE)][i] <- as.numeric((1/2) *
-            sum(diag(tcrossprod(tcrossprod(crossprod(P,
-            devV[[entries[i,1]]]), P), t(devV[[entries[i,2]]])))))
-        }
+        ranhes[lower.tri(ranhes, diag = TRUE)] <- apply(entries, 1, 
+          function(x) as.numeric((1/2) * lav_matrix_trace(tcrossprod(
+          tcrossprod(crossprod(P, devV[[x[1]]]), P), t(devV[[x[2]]])))))
       }
       if(information == "observed") {
-        for (i in 1: nrow(entries)) {
-          ranhes[lower.tri(ranhes, diag = TRUE)][i] <- -as.numeric((1/2) *
-            sum(diag(tcrossprod(tcrossprod(crossprod(P,
-            devV[[entries[i,1]]]), P), t(devV[[entries[i,2]]]))))) +
-            tcrossprod((tcrossprod((crossprod(yXbe,
-              tcrossprod(tcrossprod(crossprod(invV,
-              devV[[entries[i,1]]]), P), t(devV[[entries[i,2]]])))),
-              invV)), t(yXbe))             
-        }
+        ranhes[lower.tri(ranhes, diag = TRUE)] <- apply(entries, 1, 
+          function(x) -as.numeric((1/2) * lav_matrix_trace(
+          tcrossprod(tcrossprod(crossprod(P, devV[[x[1]]]), P), 
+          t(devV[[x[2]]])))) + tcrossprod((tcrossprod((crossprod(
+          yXbe, tcrossprod(tcrossprod(crossprod(invV,
+          devV[[x[1]]]), P), t(devV[[x[2]]])))), invV)), t(yXbe)))             
       }      
     }
-      
+    
     ranhes <- forceSymmetric(ranhes, uplo = "L")
-  
+    
     ## block 2 and block 3: second derivative of sigma and beta.
     if (information == "expected"){
-        varcov_beta <- matrix(0, length(devV), length(parts$beta))
+      varcov_beta <- matrix(0, length(devV), length(parts$beta))
     }
     if (information == "observed"){
-        varcov_beta <- matrix(NA, length(devV), length(parts$beta))
-        for (j in 1 : (length(devV))) {
-          varcov_beta[j,] <- as.vector(tcrossprod(crossprod(parts$X, 
-            (tcrossprod(crossprod(invV, devV[[j]]), invV))), t(yXbe)))
-        }
+      varcov_beta <- matrix(NA, length(devV), length(parts$beta))
+      for (j in 1 : (length(devV))) {
+        varcov_beta[j,] <- as.vector(tcrossprod(crossprod(parts$X, 
+        (tcrossprod(crossprod(invV, devV[[j]]), invV))), t(yXbe)))
+      }
     }
     ## Organize full_varcov
     full_varcov <- solve(rbind(cbind(fixhes, t(varcov_beta)),
-      cbind(varcov_beta, ranhes)))
-  
+                               cbind(varcov_beta, ranhes)))
+    
     colnames(full_varcov) <- c(names(parts$fixef), paste("cov",
-      names(parts$theta), sep="_"), "residual")
-
+                             names(parts$theta), sep="_"), "residual")
+    
     callingFun <- try(deparse(sys.call(-2)), silent = TRUE)
     if(length(callingFun) > 1) callingFun <- paste(callingFun, collapse="")
     if(!inherits(callingFun, "try-error") & grepl("summary.merMod", callingFun)){
@@ -132,4 +124,3 @@ vcov.lmerMod <- function(object, ...) {
     }
   }
 }
-
